// src/node/cli.ts
import chalk3 from "chalk";
import yargs from "yargs";
import { hideBin } from "yargs/helpers";

// src/node/build.ts
import { join } from "path";
import fs from "fs-extra";
import { resolveConfig, build as viteBuild } from "vite";
import chalk from "chalk";
async function build(cliOptions = {}) {
  const mode = process.env.MODE || process.env.NODE_ENV || cliOptions.mode || "production";
  const config = await resolveConfig({}, "build", mode);
  const cwd = process.cwd();
  const root = config.root || cwd;
  const ssgOut = join(root, ".vite-ssg-temp");
  const {
    entry = await detectEntry(root),
    onAfterClientBuild
  } = Object.assign({}, config.ssgOptions || {}, cliOptions);
  if (fs.existsSync(ssgOut))
    await fs.remove(ssgOut);
  console.log(`
${chalk.gray("[vite-ssg]")} ${chalk.yellow("Build for client...")}`);
  await viteBuild({
    build: {
      ssrManifest: true,
      rollupOptions: {
        input: {
          app: join(root, "./index.html")
        }
      }
    },
    mode: config.mode
  });
  onAfterClientBuild == null ? void 0 : onAfterClientBuild();
  console.log(`
${chalk.gray("[vite-ssg]")} ${chalk.yellow("Build for server...")}`);
  process.env.VITE_SSG = "true";
  const ssrEntry = await resolveAlias(config, entry);
  await viteBuild({
    build: {
      ssr: ssrEntry,
      outDir: ssgOut,
      minify: false,
      cssCodeSplit: false,
      rollupOptions: {
        output: {
          entryFileNames: "[name].mjs",
          format: "esm"
        }
      }
    },
    mode: config.mode
  });
}
async function detectEntry(root) {
  const scriptSrcReg = /<script(?:.*?)src=["'](.+?)["'](?!<)(?:.*)\>(?:[\n\r\s]*?)(?:<\/script>)/img;
  const html = await fs.readFile(join(root, "index.html"), "utf-8");
  const scripts = [...html.matchAll(scriptSrcReg)];
  const [, entry] = scripts.find((matchResult) => {
    const [script] = matchResult;
    const [, scriptType] = script.match(/.*\stype=(?:'|")?([^>'"\s]+)/i) || [];
    return scriptType === "module";
  }) || [];
  return entry || "src/main.ts";
}
async function resolveAlias(config, entry) {
  const resolver = config.createResolver();
  const result = await resolver(entry, config.root);
  return result || join(config.root, entry);
}

// src/node/render.ts
import { isAbsolute, join as join2, parse } from "path";
import inspector from "inspector";
import os from "os";
import chalk2 from "chalk";
import fs2 from "fs-extra";
import { resolveConfig as resolveConfig2 } from "vite";
import Piscina from "piscina";

// src/node/utils.ts
function routesToPaths(routes) {
  if (!routes)
    return ["/"];
  const paths = /* @__PURE__ */ new Set();
  const getPaths = (routes2, prefix = "") => {
    prefix = prefix.replace(/\/$/g, "");
    for (const route of routes2) {
      let path = route.path;
      if (route.path) {
        path = prefix && !route.path.startsWith("/") ? `${prefix}/${route.path}` : route.path;
        paths.add(path);
      }
      if (Array.isArray(route.children))
        getPaths(route.children, path);
    }
  };
  getPaths(routes);
  return [...paths];
}

// src/node/critical.ts
async function getCritters(outDir, options = {}) {
  try {
    const CrittersClass = (await import("critters")).default;
    const fs3 = await import("fs");
    class CrittersClassWithFS extends CrittersClass {
      constructor(...args) {
        super(...args);
        this.fs = fs3;
      }
    }
    return new CrittersClassWithFS({
      path: outDir,
      logLevel: "warn",
      external: true,
      inlineFonts: true,
      preloadFonts: true,
      ...options
    });
  } catch (e) {
    return void 0;
  }
}

// src/node/render.ts
function DefaultIncludedRoutes(paths) {
  return paths.filter((i) => !i.includes(":") && !i.includes("*"));
}
async function render(cliOptions = {}) {
  var _a;
  const mode = process.env.MODE || process.env.NODE_ENV || cliOptions.mode || "production";
  process.env.NODE_ENV = mode;
  const config = await resolveConfig2({}, "build", mode);
  const cwd = process.cwd();
  const root = config.root || cwd;
  const ssgOut = join2(root, ".vite-ssg-temp");
  const outDir = config.build.outDir || "dist";
  const out = isAbsolute(outDir) ? outDir : join2(root, outDir);
  const {
    script = "sync",
    mock = false,
    entry = await detectEntry2(root),
    formatting = "none",
    crittersOptions = {},
    includedRoutes = DefaultIncludedRoutes,
    // onBeforePageRender,
    // onPageRendered,
    onFinished,
    dirStyle = "flat",
    includeAllRoutes = false
  } = Object.assign({}, config.ssgOptions || {}, cliOptions);
  const ssrEntry = await resolveAlias2(config, entry);
  const prefix = process.platform === "win32" ? "file://" : "";
  const ssrEntryPath = join2(prefix, ssgOut, `${parse(ssrEntry).name}.mjs`);
  const { createApp } = await import(ssrEntryPath);
  const { routes } = await createApp(false);
  let routesPaths = includeAllRoutes ? routesToPaths(routes) : await includedRoutes(routesToPaths(routes));
  routesPaths = Array.from(new Set(routesPaths));
  const critters = crittersOptions !== false ? await getCritters(outDir, crittersOptions) : void 0;
  if (critters)
    console.log(`${chalk2.gray("[vite-ssg]")} ${chalk2.blue("Critical CSS generation enabled via `critters`")}`);
  if (mock) {
    const jsdomGlobal = (await import("../jsdomGlobal-34AGGKXT.mjs")).default;
    jsdomGlobal();
  }
  const ssrManifest = JSON.parse(await fs2.readFile(join2(out, "ssr-manifest.json"), "utf-8"));
  const indexHTML = await fs2.readFile(join2(ssgOut, "index.html"), "utf-8").catch(async () => {
    let indexHTML2 = await fs2.readFile(join2(out, "index.html"), "utf-8");
    indexHTML2 = rewriteScripts(indexHTML2, script);
    await fs2.writeFile(join2(ssgOut, "index.html"), indexHTML2, "utf-8");
    return indexHTML2;
  });
  const maxThreads = inspector.url() ? 1 : Math.max(1, Math.floor(Math.min(os.cpus().length / 2, os.freemem() / (1.1 * 1024 ** 3))));
  console.log(`
${chalk2.gray("[vite-ssg]")} ${chalk2.yellow("Rendering")} ${chalk2.blue(routesPaths.length)} ${chalk2.yellow("pages...")} ${chalk2.gray(`(${maxThreads} threads)`)}`);
  const pool = new Piscina({
    filename: new URL("./renderWorker.mjs", import.meta.url).pathname,
    niceIncrement: 10,
    maxThreads
  });
  await Promise.all(
    routesPaths.map((route) => {
      return pool.run({
        route,
        dirStyle,
        outDir,
        indexHTML,
        formatting,
        out,
        ssrEntryPath,
        ssrManifest
      });
    })
  );
  const pwaPlugin = (_a = config.plugins.find((i) => i.name === "vite-plugin-pwa")) == null ? void 0 : _a.api;
  if (pwaPlugin == null ? void 0 : pwaPlugin.generateSW) {
    console.log(`
${chalk2.gray("[vite-ssg]")} ${chalk2.yellow("Regenerate PWA...")}`);
    await pwaPlugin.generateSW();
  }
  console.log(`
${chalk2.gray("[vite-ssg]")} ${chalk2.green("Build finished.")}`);
  await (onFinished == null ? void 0 : onFinished());
  const waitInSeconds = 15;
  const timeout = setTimeout(() => {
    console.log(`${chalk2.gray("[vite-ssg]")} ${chalk2.yellow(`Build process still running after ${waitInSeconds}s. There might be something misconfigured in your setup. Force exit.`)}`);
    process.exit(0);
  }, waitInSeconds * 1e3);
  timeout.unref();
}
function rewriteScripts(indexHTML, mode) {
  if (!mode || mode === "sync")
    return indexHTML;
  return indexHTML.replace(/<script type="module" /g, `<script type="module" ${mode} `);
}
async function detectEntry2(root) {
  const scriptSrcReg = /<script(?:.*?)src=["'](.+?)["'](?!<)(?:.*)\>(?:[\n\r\s]*?)(?:<\/script>)/img;
  const html = await fs2.readFile(join2(root, "index.html"), "utf-8");
  const scripts = [...html.matchAll(scriptSrcReg)];
  const [, entry] = scripts.find((matchResult) => {
    const [script] = matchResult;
    const [, scriptType] = script.match(/.*\stype=(?:'|")?([^>'"\s]+)/i) || [];
    return scriptType === "module";
  }) || [];
  return entry || "src/main.ts";
}
async function resolveAlias2(config, entry) {
  const resolver = config.createResolver();
  const result = await resolver(entry, config.root);
  return result || join2(config.root, entry);
}

// src/node/cli.ts
yargs(hideBin(process.argv)).scriptName("vite-ssg").usage("$0 [args]").command(
  "build",
  "Build SSG",
  (args) => args.option("script", {
    choices: ["sync", "async", "defer", "async defer"],
    describe: "Rewrites script loading timing"
  }).option("mock", {
    type: "boolean",
    describe: "Mock browser globals (window, document, etc.) for SSG"
  }),
  async (args) => {
    await build(args);
  }
).command(
  "render",
  "Render SSG",
  (args) => args,
  async (args) => {
    await render(args);
  }
).fail((msg, err, yargs2) => {
  console.error(`
${chalk3.gray("[vite-ssg]")} ${chalk3.red.bold("An internal error occurred.")}`);
  console.error(`${chalk3.gray("[vite-ssg]")} ${chalk3.white(`Please report an issue, if none already exists: ${chalk3.underline("https://github.com/antfu/vite-ssg/issues")}`)}`);
  yargs2.exit(1, err);
}).showHelpOnFail(false).help().argv;
