// src/node/renderWorker.ts
import { dirname, join } from "path";
import chalk from "chalk";
import { renderToString } from "vue/server-renderer";
import { JSDOM } from "jsdom";
import fs from "fs-extra";

// src/node/preload-links.ts
function renderPreloadLinks(document, modules, ssrManifest) {
  const seen = /* @__PURE__ */ new Set();
  const preloadLinks = [];
  Array.from(modules).forEach((id) => {
    const files = ssrManifest[id] || [];
    files.forEach((file) => {
      if (!preloadLinks.includes(file))
        preloadLinks.push(file);
    });
  });
  if (preloadLinks) {
    preloadLinks.forEach((file) => {
      if (!seen.has(file)) {
        seen.add(file);
        renderPreloadLink(document, file);
      }
    });
  }
}
function renderPreloadLink(document, file) {
  if (file.endsWith(".js")) {
    appendLink(document, {
      rel: "modulepreload",
      crossOrigin: "",
      href: file
    });
  } else if (file.endsWith(".css")) {
    appendLink(document, {
      rel: "stylesheet",
      href: file
    });
  }
}
var createLink = (document) => document.createElement("link");
var setAttrs = (el, attrs) => {
  const keys = Object.keys(attrs);
  for (const key of keys)
    el.setAttribute(key, attrs[key]);
};
function appendLink(document, attrs) {
  const exits = document.head.querySelector(`link[href='${attrs.file}']`);
  if (exits)
    return;
  const link = createLink(document);
  setAttrs(link, attrs);
  document.head.appendChild(link);
}

// src/node/renderWorker.ts
var renderWorker_default = async ({ route, dirStyle, outDir, indexHTML, formatting, out, ssrEntryPath, ssrManifest }) => {
  try {
    const { createApp } = await import(ssrEntryPath);
    const relativeRouteFile = `${(route.endsWith("/") ? `${route}index` : route).replace(/^\//g, "")}.html`;
    const filename = dirStyle === "nested" ? join(route.replace(/^\//g, ""), "index.html") : relativeRouteFile;
    console.log(
      `Page ${chalk.dim(`${outDir}/`)}${chalk.cyan(filename.padEnd(15, " "))}`
    );
    const appCtx = await createApp(false, route);
    const { app, router, head, initialState } = appCtx;
    if (router) {
      await router.push(route);
      await router.isReady();
    }
    const transformedIndexHTML = (
      /* (await onBeforePageRender?.(route, indexHTML, appCtx)) || */
      indexHTML
    );
    const ctx = {};
    const appHTML = await renderToString(app, ctx);
    const renderedHTML = renderHTML({ indexHTML: transformedIndexHTML, appHTML, initialState });
    const jsdom = new JSDOM(renderedHTML);
    renderPreloadLinks(jsdom.window.document, ctx.modules || /* @__PURE__ */ new Set(), ssrManifest);
    head == null ? void 0 : head.updateDOM(jsdom.window.document);
    const html = jsdom.serialize();
    const transformed = (
      /* (await onPageRendered?.(route, html, appCtx)) || */
      html
    );
    const formatted = await format(transformed, formatting);
    await fs.ensureDir(join(out, dirname(filename)));
    await fs.writeFile(join(out, filename), formatted, "utf-8");
    delete app.config.errorHandler;
    delete app.config.globalProperties;
    delete app._context.provides;
    delete app._context.components;
    delete app._context.mixins;
    delete app._context.config;
    delete app._context;
    delete app._component;
  } catch (err) {
    console.error(`${chalk.gray("[vite-ssg]")} ${chalk.red(`Error on page: ${chalk.cyan(route)}`)}
${err.stack}`);
  }
};
function renderHTML({ indexHTML, appHTML, initialState }) {
  const stateScript = initialState ? `
<script>window.__INITIAL_STATE__=${initialState}</script>` : "";
  return indexHTML.replace(
    '<div id="app"></div>',
    `<div id="app" data-server-rendered="true">${appHTML}</div>${stateScript}`
  );
}
async function format(html, formatting) {
  if (formatting === "minify") {
    const htmlMinifier = await import("html-minifier");
    return htmlMinifier.minify(html, {
      collapseWhitespace: true,
      caseSensitive: true,
      collapseInlineTagWhitespace: false,
      minifyJS: true,
      minifyCSS: true
    });
  } else if (formatting === "prettify") {
    const prettier = (await import("prettier/esm/standalone.mjs")).default;
    const parserHTML = (await import("prettier/esm/parser-html.mjs")).default;
    return prettier.format(html, { semi: false, parser: "html", plugins: [parserHTML] });
  }
  return html;
}
export {
  renderWorker_default as default
};
